/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.lang.reflect.InvocationTargetException;

import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.Domain;
import org.ow2.petals.topology.generated.JmxService;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.SubdomainMode;
import org.ow2.petals.topology.generated.Topology;
import org.ow2.petals.topology.generated.TransportService;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Utility class for unit test about the configuration of a Petals ESB node.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ConfigurationTestUtils {

    public final static String DOMAIN_NAME = "PEtALS";

    public final static String DOMAIN_DESCRIPTION = "A sample domain configuration";

    public final static String SUBDOMAIN_NAME_0 = "subdomain-0";

    public final static SubdomainMode SUBDOMAIN_MODE_0 = SubdomainMode.STATIC;

    public final static String SUBDOMAIN_DESCRIPTION_0 = "A sample sub-domain 0 configuration";

    public final static String CONTAINER_NAME_0 = "sample-0";

    public final static String CONTAINER_DESCRIPTION_0 = "description of the container 0";

    public final static int CONTAINER_TCP_PORT_0 = 7800;

    public final static String CONTAINER_HOST = "localhost";

    public final static String CONTAINER_JMX_USER = "petals";

    public final static String CONTAINER_JMX_PWD = "petals";

    public final static String TOPOLOGY_PASSPHRASE = "topo-passphrase";

    private ConfigurationTestUtils() {
        // No constructor in utility class
    }

    /**
     * Create a sample topology with only required parameters
     * 
     * @return A sample topology with only required parameters
     */
    public static Topology createBasicSampleTopology() {
        final Topology topology = new Topology();
        final Domain domain = new Domain();
        topology.setDomain(domain);

        domain.setName(DOMAIN_NAME);
        domain.setDescription(DOMAIN_DESCRIPTION);

        final Subdomain subdomain = new Subdomain();
        domain.getSubDomain().add(subdomain);
        subdomain.setName(SUBDOMAIN_NAME_0);
        subdomain.setMode(SUBDOMAIN_MODE_0);
        subdomain.setDescription(SUBDOMAIN_DESCRIPTION_0);

        final Container container0 = new Container();
        subdomain.getContainer().add(container0);
        container0.setName(CONTAINER_NAME_0);
        container0.setDescription(CONTAINER_DESCRIPTION_0);
        container0.setHost(CONTAINER_HOST);
        container0.setUser(CONTAINER_JMX_USER);
        container0.setPassword(CONTAINER_JMX_PWD);

        final JmxService jmxService0 = new JmxService();
        container0.setJmxService(jmxService0);

        final TransportService transportService0 = new TransportService();
        transportService0.setTcpPort(CONTAINER_TCP_PORT_0);
        container0.setTransportService(transportService0);

        return topology;
    }

    /**
     * Assert a basic sample topology, for example created by {@link #createBasicSampleTopology()}
     * 
     * @param topology
     *            The topology to check
     */
    public static void assertBasicSampleTopology(final Topology topology) {

        assertNotNull("Topology is null", topology);

        final Domain domain = topology.getDomain();
        assertNotNull("Domain is null", domain);
        assertEquals("Unexpected domain name", ConfigurationTestUtils.DOMAIN_NAME, domain.getName());
        assertEquals("Unexpected domain description", ConfigurationTestUtils.DOMAIN_DESCRIPTION,
                domain.getDescription());

        assertNotNull("Sub-domain list null", domain.getSubDomain());
        assertEquals("Unexpected number of sub-domain", 1, domain.getSubDomain().size());
        final Subdomain subdomain = domain.getSubDomain().get(0);
        assertEquals("Unexpected sub-domain name", ConfigurationTestUtils.SUBDOMAIN_NAME_0, subdomain.getName());
        assertEquals("Unexpected sub-domain description", ConfigurationTestUtils.SUBDOMAIN_DESCRIPTION_0,
                subdomain.getDescription());
        assertEquals("Unexpected sub-domain mode", ConfigurationTestUtils.SUBDOMAIN_MODE_0, subdomain.getMode());

        assertNotNull("Container list null", subdomain.getContainer());
        assertEquals("Unexpected number of container", 1, subdomain.getContainer().size());
        final Container container = subdomain.getContainer().get(0);
        assertEquals("Unexpected container name", ConfigurationTestUtils.CONTAINER_NAME_0, container.getName());
        assertEquals("Unexpected container description", ConfigurationTestUtils.CONTAINER_DESCRIPTION_0,
                container.getDescription());
        assertEquals("Unexpected container host", ConfigurationTestUtils.CONTAINER_HOST, container.getHost());
        assertEquals("Unexpected container jmx user", ConfigurationTestUtils.CONTAINER_JMX_USER, container.getUser());
        assertEquals("Unexpected container jmx password", ConfigurationTestUtils.CONTAINER_JMX_PWD,
                container.getPassword());

        assertNotNull("JMX service null", container.getJmxService());
        assertNotNull("TCP service null", container.getTransportService());
        assertEquals("Unexpected container TCP port", ConfigurationTestUtils.CONTAINER_TCP_PORT_0, container
                .getTransportService().getTcpPort());

    }

    /**
     * Start a service 'configuration'
     * 
     * @param cfgSvc
     *            The service 'configuration' to start
     * @throws InvocationTargetException
     *             An error occurs during the start. Should not occur
     */
    public static void startConfigurationService(final ConfigurationService cfgSvc) throws InvocationTargetException {
        ReflectionHelper.invokePrivateMethod(cfgSvc, "start", null, null);
    }

    /**
     * Stop a service 'configuration'
     * 
     * @param cfgSvc
     *            The service 'configuration' to stop
     * @throws InvocationTargetException
     *             An error occurs during the stop. Should not occur
     */
    public static void stopConfigurationService(final ConfigurationService cfgSvc) throws InvocationTargetException {
        ReflectionHelper.invokePrivateMethod(cfgSvc, "stop", null, null);
    }
}
